"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ts_simple_ast_1 = require("ts-simple-ast");
const path_1 = require("path");
const fs_1 = require("fs");
const util_1 = require("./util");
/**
 * Low level public API to check type of `value` match type `typeOrFunction` in the context of project `tsConfigFilePath`.
 */
function checkType(typeOrFunction, value, options = {}) {
    return checkTypeCore(typeOrFunction, value, options);
}
exports.checkType = checkType;
/** @internal */
function checkTypeCore(typeOrFunction, value, options = {}) {
    options.dontEscape = options.asString || options.dontEscape;
    options.printResult && console.log(options);
    let d;
    let sourceFile;
    const tsConfigFilePath = options.tsConfigFilePath || './tsconfig.json';
    if (!project) {
        project = new ts_simple_ast_1.Project({
            tsConfigFilePath,
            addFilesFromTsConfig: true
        });
    }
    const { callerFile, allCallerFiles } = util_1.getCallerFile();
    if (!callerFile) {
        return {
            pass: false,
            error: `Caller source file cannot be found, aborting`,
            allCallerFiles
        };
    }
    const callerSourceFile = project.getSourceFile(callerFile);
    if (!callerSourceFile || !fs_1.existsSync(callerFile)) {
        return {
            pass: false,
            error: `Caller source file must belong to "${tsConfigFilePath}" project and exist but "${callerFile}" does not`,
            callerFile,
            allCallerFiles
        };
    }
    if (options.verifyProject) {
        d = project.getPreEmitDiagnostics();
        if (d.length) {
            return {
                pass: false,
                error: `Given TypeScript project cannot have compilation errors since option verifyProject was given. Fix them and try again. Errors: ${util_1.formatDiagnostics(d)}`,
                callerFile,
                allCallerFiles
            };
        }
    }
    else if (!options.dontVerifyFile) {
        d = callerSourceFile.getPreEmitDiagnostics();
        if (d.length) {
            return {
                pass: false,
                error: `Caller TypeScript file cannot have compilation errors, fix them and try again or pass option 'options.dontVerifyFile'. Errors: ${util_1.formatDiagnostics(d)}`,
                callerFile,
                allCallerFiles
            };
        }
    }
    const folderName = path_1.dirname(callerFile);
    const ext = path_1.extname(callerFile);
    const fileName = `${options.folder || ''}${util_1.unique(path_1.basename(callerFile, ext))}.${ext}`;
    const filePath = path_1.join(folderName, fileName);
    let testCode;
    const v = value &&
        typeof value.__tsdCR_prefix === 'string' &&
        typeof value.text === 'string'
        ? value.text
        : value;
    const escapedValue = options.dontEscape ? v : util_1.escapeValue(v, options);
    if (escapedValue === undefined) {
        return {
            pass: false,
            error: `Value is not valid JSON and option enforceJsonValues was used. Value: ${v}`,
            callerFile,
            filePath,
            allCallerFiles
        };
    }
    if (typeof typeOrFunction === 'function') {
        typeOrFunction = typeOrFunction(escapedValue);
        options.dontCreateTestCodeVariable = true;
    }
    let { text, __tsdCR_prefix: prefix } = typeof typeOrFunction === 'string' ? { text: typeOrFunction, __tsdCR_prefix: '' } : typeOrFunction;
    if (!prefix && value && typeof value.__tsdCR_prefix === 'string') {
        prefix = value.__tsdCR_prefix;
    }
    if (options.dontCreateTestCodeVariable) {
        testCode = `${prefix}\n${text}`;
    }
    else {
        testCode = `${prefix}\nconst ${util_1.unique('variable')}: ${text} = ${escapedValue}`;
    }
    const code = `
${fs_1.readFileSync(callerFile).toString()}
function ${util_1.unique('__checkType')}(){
${testCode}
}`;
    sourceFile = project.createSourceFile(filePath, code);
    d = sourceFile.getPreEmitDiagnostics();
    const failErrors = util_1.formatDiagnostics(d);
    const pass = d.length === 0
        ? true
        : options.failOnlyWithErrorCodes
            ? !failErrors.find(e => options.failOnlyWithErrorCodes.includes(e.code))
            : false;
    const r = {
        pass,
        failErrors,
        code,
        testCode,
        callerFile,
        filePath,
        allCallerFiles
    };
    if (options.printResult || (!r.pass && options.printResultIfFail)) {
        console.log(JSON.stringify(Object.assign({}, r, { code: null }), null, 2));
    }
    return r;
}
exports.checkTypeCore = checkTypeCore;
let project;
